#Imports
import json
from PIL import Image, ImageOps
from numpy import asarray 
import numpy as np

class Dataset(object):
	'''
        A class for the dataset that will return data items as per the given index
	'''

	def __init__(self, annotation_file, transforms = None):
		'''
            Arguments:
            annotation_file: path to the annotation file
            transforms: list of transforms (class instances)
                        For instance, [<class 'RandomCrop'>, <class 'Rotate'>]
		'''
		
		self.annotation_file = annotation_file
		
		if transforms==None:
			self.transforms = []
		else:
			self.transforms = transforms

		self.dataPoints = []
		fin = open(self.annotation_file,"r")
		
		for line in fin:
			self.dataPoints.append(json.loads(line.strip('\n')))

		fin.close()


	def __len__(self):
		'''
		    return the number of data points in the dataset
		'''
		return self.dataPoints.size
        

	def __getitem__(self, idx):
		'''
			return the dataset element for the index: "idx"
			Arguments:
				idx: index of the data element.

			Returns: A dictionary with:
				image: image (in the form of a numpy array) (shape: (3, H, W))
				gt_bboxes: N X 5 array where N is the number of bounding boxes, each 
							consisting of [class, x1, y1, x2, y2]
							x1 and x2 lie between 0 and width of the image,
							y1 and y2 lie between 0 and height of the image.

			You need to do the following, 
			1. Extract the correct annotation using the idx provided.
			2. Read the image and convert it into a numpy array (wont be necessary
				with some libraries). The shape of the array would be (3, H, W).
			3. Scale the values in the array to be with [0, 1].
			4. Create a dictonary with both the image and annotations
			4. Perform the desired transformations.
			5. Return the transformed image and annotations as specified.
		'''
		img_path = self.dataPoints[idx]['img_fn']
		img = np.array(Image.open("./data/" + img_path))

		# print(type(img))
		for transform in self.transforms:
			img = transform(img)
		
		print(type(img))
		img = img.transpose(2, 0, 1).astype('float')
		img /= 255

		ret = {}
		ret['image'] = img
		ret['bboxes'] = self.dataPoints[idx]['bboxes']
		
		return ret

# dataset = Dataset("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/data/annotations.jsonl")
# Image.fromarray((dataset[0]['image']*255).transpose(2,1,0).astype(np.uint8)).show()



#Imports
from PIL import ImageDraw, Image, ImageFont

def plot_boxes(image, path, boxes, filename): # Write the required arguments

  # The function should plot the predicted boxes on the images and save them.
  # Tip: keep the dimensions of the output image less than 800 to avoid RAM crashes.
  
  font = ImageFont.truetype("Pillow/Tests/fonts/FreeMono.ttf", 25)
  image = Image.fromarray(image)
  image_d = ImageDraw.Draw(image)
  for i, box in enumerate(boxes[:5]):
    box_t = tuple(box)
    image_d.rectangle(box_t, outline="blue")
    image_d.text((box[1][0], box[1][1]), filename[i], fill = 'red', font = font, stroke_width = 1)
  pth = path
  image.save(fp = pth)
  return image


#Imports
from PIL import Image
import numpy as np

class RotateImage(object):
    '''
        Rotates the image about the centre of the image.
    '''

    def __init__(self, degrees):
        '''
            Arguments:
            degrees: rotation degree.
        '''
        
        # Write your code here
        self.degrees = degrees

    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''

        # Write your code here
        return np.array(Image.fromarray(image).rotate(self.degrees, Image.NEAREST, expand = 1))

# img = Image.open("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/sample_imgs/bbox.png")
        
# rotator = RotateImage(45)
# rotatedImage = Image.fromarray(rotator(np.array(img)))
# rotatedImage.show()

# rotator = RotateImage(90)
# rotatedImage = Image.fromarray(rotator(np.array(img)))
# rotatedImage.show()



#Imports
from  PIL import Image
import numpy as np
from random import randint

class CropImage(object):
    '''
        Performs either random cropping or center cropping.
    '''

    def __init__(self, shape, crop_type='center'):
        '''
            Arguments:
            shape: output shape of the crop (h, w)
            crop_type: center crop or random crop. Default: center
        '''
        
        # Write your code here
        self.shape = shape
        self.crop_type = crop_type


    # def cropCenter(self,pil_img):
    #     img_width, img_height = pil_img.size
    #     return pil_img.crop(((img_width - self.shape.second) // 2,
    #                         (img_height - self.shape.first) // 2,
    #                         (img_width + self.shape.second) // 2,
    #                         (img_height + self.shape.second) // 2))


    def __call__(self, image):
        '''
            Arguments:
            image (numpy array or PIL image)

            Returns:
            image (numpy array or PIL image)
        '''
        # Write your code here

        if self.crop_type == "center":
            img_width, img_height = Image.fromarray(image).size
            return np.array(Image.fromarray(image).crop(((img_width - self.shape[1]) // 2,
                            (img_height - self.shape[0]) // 2,
                            (img_width + self.shape[1]) // 2,
                            (img_height + self.shape[0]) // 2)))

        elif self.crop_type == "random":
            img_width, img_height = Image.fromarray(image).size
            new_height, new_width = self.shape
            left = randint(0, img_width-new_width)
            top = randint(0, img_height-new_height)
            right = left + new_width
            bottom = top + new_height
            return np.array(Image.fromarray(image).crop((left,top,right,bottom)))
            # assert img.shape[0] >= height
            # assert img.shape[1] >= width
            # assert img.shape[0] == mask.shape[0]
            # assert img.shape[1] == mask.shape[1]
            # x = random.randint(0, img.shape[1] - width)
            # y = random.randint(0, img.shape[0] - height)
            # img = img[y:y+height, x:x+width]
            # mask = mask[y:y+height, x:x+width]
            # return img, mask


# img = Image.open("/home/anish/Software Development Lab/Assignment-3/CS29006_SW_Lab_Spr2021/Python_DS_Assignment/AssignmentQs2/sample_imgs/bbox.png")
        

# Cropper = CropImage((250,250))

# croppedImage = Image.fromarray(Cropper(np.array(img)))

# croppedImage.show()

# a = (5,10)
# print(a[0])